#[non_exhaustive]pub struct ModeChange<M> {
pub target: M,
}Expand description
A requested change from one mode to another.
Produced inside a Decision when an event triggers a mode transition.
The runtime inspects this value to determine whether the system should
move to a new mode.
Examples
use ready_active_safe::Decision;
let d: Decision<&str, (), ()> = Decision::transition("active");
let change = d.mode_change();
assert_eq!(change.map(|mc| mc.target), Some("active"));Fields (Non-exhaustive)
This struct is marked as non-exhaustive
Non-exhaustive structs could have additional fields added in future. Therefore, non-exhaustive structs cannot be constructed in external crates using the traditional
Struct { .. } syntax; cannot be matched against without a wildcard ..; and struct update syntax will not work.target: MThe mode to transition to.