pub struct Instant(/* private fields */);Available on crate feature
time only.Expand description
A monotonic instant represented as nanoseconds since an arbitrary origin.
This type is intentionally small and no_std friendly. It is suitable for:
- deadlines (
Deadline) - timeouts and elapsed-time calculations
- deterministic replay (when driven by a virtual clock)
Implementations
Sourceimpl Instant
impl Instant
Sourcepub const fn from_nanos(nanos: u64) -> Self
pub const fn from_nanos(nanos: u64) -> Self
Creates an Instant from a nanosecond counter.
Sourcepub const fn as_nanos(self) -> u64
pub const fn as_nanos(self) -> u64
Returns this instant as nanoseconds since an arbitrary origin.
Sourcepub const fn checked_duration_since(self, earlier: Self) -> Option<Duration>
pub const fn checked_duration_since(self, earlier: Self) -> Option<Duration>
Returns the duration between earlier and self, if self >= earlier.
Sourcepub fn checked_add(self, duration: Duration) -> Option<Self>
pub fn checked_add(self, duration: Duration) -> Option<Self>
Returns the result of adding duration to this instant, or None on overflow.
Sourcepub fn checked_sub(self, duration: Duration) -> Option<Self>
pub fn checked_sub(self, duration: Duration) -> Option<Self>
Returns the result of subtracting duration from this instant, or None on underflow.