SystemClock

Struct SystemClock 

pub struct SystemClock { /* private fields */ }
Available on crate features time and std only.
A monotonic system clock.

This clock requires std and measures time as “elapsed since creation”. It is appropriate for deadlines and timeouts where only monotonic progress matters.

Implementations

Source

impl SystemClock

pub fn new() -> Self

Creates a new system clock.