macro_rules! assert_stays {
($decision:expr) => { ... };
}Expand description
Asserts that a decision stays in the current mode.
Produces a clear failure message when the assertion fails.
Examples
use ready_active_safe::prelude::*;
use ready_active_safe::assert_stays;
let d: Decision<&str, ()> = stay();
assert_stays!(d);