pub struct SystemClock { /* private fields */ }Available on crate features
time and std only.Expand description
A monotonic system clock.
This clock requires std and measures time as “elapsed since creation”.
It is appropriate for deadlines and timeouts where only monotonic progress
matters.
Implementations
Sourceimpl SystemClock
impl SystemClock